==> spellathon-5.47.tar.gz: spellathon-5.47/src/gui/wordinput.cc <==
/*
 * Copyright (c) 2009, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * The names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#include <QObject>
#include <QWidget>
#include <QLineEdit>
#include <QPainter>
#include <QKeyEvent>
#include <QPaintEvent>
#include <QMouseEvent>
#include <QTimer>

#include "wordinput.h"

WordInput::WordInput(QWidget *p) 
	: QLineEdit(p), chk_state(BLANK), init_state(true), timer(0) 
{ 
	setContextMenuPolicy(Qt::NoContextMenu); 
	timer = new QTimer(this);
	timer->setSingleShot(true);
	timer->setInterval(2000);
	connect(timer, SIGNAL(timeout()), this, SLOT(chk_clear()));	
}

void 
WordInput::setInitState(bool s) 
{ 
	init_state = s; 
	ip_state = false;
	timer->stop();
	chk_state = 2;
	clear();
	if (init_state) {
		setStyleSheet("color: grey; font: italic");
		setText("Enter a word");
		setCursorPosition(0);
	} else {
		setStyleSheet("color: black;");
	}
}

void 
WordInput::paintEvent(QPaintEvent *ev) 
{
	QLineEdit::paintEvent(ev);
	QPainter r(this);

	switch (chk_state) {
	case REPEAT:
		r.drawPixmap(size().width()-(16+8), (size().height()-16)/2, 
					 QPixmap(QString(":/repeat.png")));
		break;
	case CORRECT:
		r.drawPixmap(size().width()-(16+8), (size().height()-16)/2, 
					 QPixmap(QString(":/tick.png")));
		break;
	case WRONG:
		r.drawPixmap(size().width()-(16+8), (size().height()-16)/2, 
					 QPixmap(QString(":/cross.png")));
		break;
	case BLANK:
		break;
	}
}

void 
WordInput::mousePressEvent(QMouseEvent *ev)
{
	if (init_state) {
		setInitState(false);
	} else if (ip_state) {
		setIpState(false);
	} else {
		QLineEdit::mousePressEvent(ev);
	}
	ev->accept();
}

void 
WordInput::keyPressEvent(QKeyEvent *ev)
{
	int k = ev->key();
	if (init_state) {
		if ((k >= Qt::Key_A && k <= Qt::Key_Z)) {
			setInitState(false);
			QLineEdit::keyPressEvent(ev);
		}
	} else if (ip_state) {
		if ((k >= Qt::Key_A && k <= Qt::Key_Z)) {
			setIpState(false);
			QLineEdit::keyPressEvent(ev);
		}
	} else {
		if (k == Qt::Key_Return || k == Qt::Key_Enter) {
			if (text() != "") {
				setIpState(true);
				QLineEdit::keyPressEvent(ev);
			}
		} else {
				QLineEdit::keyPressEvent(ev);
		}
	}
	ev->accept();
}

void
WordInput::setIpState(bool e)
{
	if (e) {
		setCursorPosition(0);
		setStyleSheet("color: #606060;");
		ip_state = true;
	} else {
		ip_state = false;
		chk_state = BLANK;
		clear();
		timer->stop();
		setStyleSheet("color: black;");
	}
}

void 
WordInput::focusInEvent(QFocusEvent *ev)
{
	QLineEdit::focusInEvent(ev);
	if (init_state)
		setCursorPosition(0);
}

void 
WordInput::chk_clear() 
{ 
	chk_state = BLANK; 
	clear();
	setStyleSheet("color: black;");
}

void 
WordInput::chk(int v)
{
	chk_state = v;
	switch (v) {
	case CORRECT:
		setStyleSheet("padding-right: 28; color: #606060; border-width: 1px;"
					  "border-style: inset; border-color: darkgreen;");
		break;
	case WRONG:
		setStyleSheet("padding-right: 28; color: #606060;  border-width: 1px;"
					  "border-style: inset; border-color: darkred;");
		break;
	case REPEAT:
		setStyleSheet("padding-right: 28; color: #606060;");
		break;
	}

	timer->start();
}

==> spellathon-5.53.tar.gz: spellathon-5.53/src/gui/wordinput.cc <==
/*
 * Copyright (c) 2009, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * The names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#include <QObject>
#include <QWidget>
#include <QLineEdit>
#include <QPainter>
#include <QKeyEvent>
#include <QPaintEvent>
#include <QMouseEvent>
#include <QTimer>

#include "wordinput.h"

#define BASE_STYLE "border: 1px inset #787878; border-radius: 3px; background: white;"\
				   "selection-background-color: darkgray; padding: 1 1px;"

WordInput::WordInput(QWidget *p) 
	: QLineEdit(p), chk_state(BLANK), init_state(true), timer(0) 
{ 
	setContextMenuPolicy(Qt::NoContextMenu); 
	timer = new QTimer(this);
	timer->setSingleShot(true);
	timer->setInterval(2000);
	connect(timer, SIGNAL(timeout()), this, SLOT(chk_clear()));	
}

void 
WordInput::setInitState(bool s) 
{ 
	init_state = s; 
	ip_state = false;
	timer->stop();
	chk_state = 2;
	clear();
	if (init_state) {
		setStyleSheet(BASE_STYLE "color: grey; font: italic");
		setText("Enter a word");
		setCursorPosition(0);
	} else {
		setStyleSheet(BASE_STYLE "color: black");
	}
}

void 
WordInput::paintEvent(QPaintEvent *ev) 
{
	QLineEdit::paintEvent(ev);
	QPainter r(this);

	switch (chk_state) {
	case REPEAT:
		r.drawPixmap(size().width()-(16+8), (size().height()-16)/2, 
					 QPixmap(QString(":/repeat.png")));
		break;
	case CORRECT:
		r.drawPixmap(size().width()-(16+8), (size().height()-16)/2, 
					 QPixmap(QString(":/tick.png")));
		break;
	case WRONG:
		r.drawPixmap(size().width()-(16+8), (size().height()-16)/2, 
					 QPixmap(QString(":/cross.png")));
		break;
	case BLANK:
		break;
	}
}

void 
WordInput::mousePressEvent(QMouseEvent *ev)
{
	if (init_state) {
		setInitState(false);
	} else if (ip_state) {
		setIpState(false);
	} else {
		QLineEdit::mousePressEvent(ev);
	}
	ev->accept();
}

void 
WordInput::keyPressEvent(QKeyEvent *ev)
{
	int k = ev->key();
	if (init_state) {
		if ((k >= Qt::Key_A && k <= Qt::Key_Z)) {
			setInitState(false);
			QLineEdit::keyPressEvent(ev);
		}
	} else if (ip_state) {
		if ((k >= Qt::Key_A && k <= Qt::Key_Z)) {
			setIpState(false);
			QLineEdit::keyPressEvent(ev);
		}
	} else {
		if (k == Qt::Key_Return || k == Qt::Key_Enter) {
			if (text() != "") {
				setIpState(true);
				QLineEdit::keyPressEvent(ev);
			}
		} else {
				QLineEdit::keyPressEvent(ev);
		}
	}
	ev->accept();
}

void
WordInput::setIpState(bool e)
{
	if (e) {
		setCursorPosition(0);
		setStyleSheet(BASE_STYLE "color: #606060");
		ip_state = true;
	} else {
		ip_state = false;
		chk_state = BLANK;
		clear();
		timer->stop();
		setStyleSheet(BASE_STYLE "color: black");
	}
}

void 
WordInput::focusInEvent(QFocusEvent *ev)
{
	QLineEdit::focusInEvent(ev);
	if (init_state)
		setCursorPosition(0);
}

void 
WordInput::chk_clear() 
{ 
	chk_state = BLANK; 
	clear();
	setStyleSheet(BASE_STYLE "color: black");
}

void 
WordInput::chk(int v)
{
	chk_state = v;
	switch (v) {
	case CORRECT:
		setStyleSheet(BASE_STYLE "padding-right: 28; color: #606060;"
					 " border-color: darkgreen");
		break;
	case WRONG:
		setStyleSheet(BASE_STYLE "padding-right: 28; color: #606060;"
					  "border-color: darkred;");
		break;
	case REPEAT:
		setStyleSheet(BASE_STYLE "padding-right: 28; color: #606060;");
		break;
	}

	timer->start();
}
